"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bot = void 0;
// Modules.
const DotEnv = require("dotenv");
const ClientFactory_1 = require("../Client/ClientFactory");
const ClientType_1 = require("../Client/ClientType");
const Akechi_1 = require("../Confidant/Akechi");
const Morgana_1 = require("../Confidant/Morgana");
const Sojiro_1 = require("../Confidant/Sojiro");
/**
 * Provides a class for Bots.
 *
 * Bots are the fruit of this application. They're the whole point of it. And this is where it all happens!
 *
 * Configuration for bots are managed in a 'config.yml' file found in their folder. From there, functions in here
 * manage the authentication to the bot's clients and what talents the bot has.
 */
class Bot {
    /**
     * Bot constructor.
     *
     * @param id
     *   ID of the bot. This is the name of the folder, not a reader-friendly name.
     * @param config
     *   Configuration loaded from the bot's 'NAME.config.yml' file.
     * @param directory
     *   Path to the directory where this bot's files are stored.
     */
    constructor(id, config, directory) {
        /**
         * Object to store list of Clients the bot has.
         */
        this.clients = {};
        /**
         * Array to store a list of all Prompts attached to this bot.
         */
        this.prompts = [];
        /**
         * Object to store data about the bot's master user.
         */
        this.joker = {};
        /**
         * Boolean to determine whether the bot is set to maintenance mode or not.
         */
        this.maintenance = false;
        /**
         * Boolean to determine if the bot is the Master Bot. There can only be one!
         */
        this.isMaster = false;
        /**
         * Boolean to store whether or no the bot is summoned.
         */
        this.summoned = false;
        this.id = id;
        this.config = config;
        this.directory = directory;
    }
    /**
     * Synthesis handler for a Bot.
     *
     * The BotManager service will run all synthesis handlers for bots during the Synthesis phase.
     *
     * Initializes clients, talents, commands and listeners.
     */
    synthesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Load environment variables.
            yield this.loadEnvironmentVariables()
                .catch(() => __awaiter(this, void 0, void 0, function* () {
                yield Morgana_1.Morgana.error(`Could not load environent variables for ${this.id}. Is the .env created for this bot?`);
            }));
        });
    }
    /**
     * Summoning handler for this Bot.
     *
     * Authenticates the clients and initializes talents.
     */
    summon() {
        return __awaiter(this, void 0, void 0, function* () {
            // If the bot is already summoned, we don't want to do anything here.
            if (this.summoned) {
                yield Morgana_1.Morgana.warn("Tried to summon {{bot}}, but the bot is already summoned!", { bot: this.id });
                return;
            }
            // If the environment variables aren't set, we can't do anything here.
            if (Sojiro_1.Sojiro.isEmpty(this.env)) {
                yield Morgana_1.Morgana.error("Tried to summon '{{bot}}', but the bot has no environment variables set!", { bot: this.id });
                yield Morgana_1.Morgana.error("Make sure a .env file exists for {{bot}}!", { bot: this.id });
                return;
            }
            // Await client initialization.
            yield this.initializeClients();
            // Await clients authentication.
            yield this.authenticateClients();
            // Await building of architect.
            yield this.setJoker();
            // Set the bot's summoned flag to true.
            this.summoned = true;
        });
    }
    /**
     * Shutdown the bot, disconnecting it from all clients.
     */
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            // If the bot isn't summoned, we can't shut it down.
            if (!this.summoned) {
                yield Morgana_1.Morgana.warn("Tried to shutdown {{bot}}, but it's already disconnected!", { bot: this.id });
                return;
            }
            // Disconnect the bot from all clients.
            yield this.disconnectClients();
            // Set the bot's summoned flag to true.
            this.summoned = false;
        });
    }
    /**
     * Retrieve a specific client from a Bot.
     *
     * @param clientType
     *   The type of client to return from the bot.
     *
     * @returns
     *   The requested client.
     */
    getClient(clientType) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.clients[clientType];
        });
    }
    /**
     * Retrieve configuration for a specific client in a bot.
     *
     * @param clientType
     *   The type of client configuration to return for the bot.
     *
     * @returns
     *   The requested client configuration from the base files.
     */
    getClientConfig(clientType) {
        return __awaiter(this, void 0, void 0, function* () {
            // Determine path to client configuration.
            const pathToClientConfig = `${this.directory}/clients/${clientType}.yml`;
            // Attempt to fetch client configuration.
            if (!(yield Akechi_1.Akechi.fileExists(pathToClientConfig))) {
                return undefined;
            }
            // Load configuration since it exists.
            return yield Akechi_1.Akechi.readYamlFile(pathToClientConfig);
        });
    }
    /**
     * Remove a prompt from the current bot.
     *
     * @param prompt
     *   The prompt to remove from this bot.
     */
    removePrompt(prompt) {
        return __awaiter(this, void 0, void 0, function* () {
            this.prompts = Sojiro_1.Sojiro.removeFromArray(this.prompts, prompt);
        });
    }
    /**
     * Disconnect all of the clients in this bot.
     */
    disconnectClients() {
        return __awaiter(this, void 0, void 0, function* () {
            // Await the authentication of the clients linked to the bot.
            for (const key of Object.keys(this.clients)) {
                // Await authentication of the bot.
                yield this.disconnectClient(ClientType_1.ClientType[key]);
            }
        });
    }
    /**
     * Disconnect from a determined client on this bot.
     *
     * @param clientType
     *   The client ID to disconnect from.
     */
    disconnectClient(clientType) {
        return __awaiter(this, void 0, void 0, function* () {
            // Simply call the client's disconnect function.
            const client = yield this.getClient(clientType);
            yield client.disconnect();
        });
    }
    /**
     * Load the .env file specific to this bot, and parse its contents.
     */
    loadEnvironmentVariables() {
        return __awaiter(this, void 0, void 0, function* () {
            const envFileData = yield Akechi_1.Akechi.readFile(`${this.directory}/.env`);
            this.env = DotEnv.parse(envFileData);
        });
    }
    /**
     * For each client, we build Joker's identification and data.
     *
     * We should be able to access Joker's information from the bot at all times.
     */
    setJoker() {
        return __awaiter(this, void 0, void 0, function* () {
            // Await processing of all clients.
            for (const [key, client] of Object.entries(this.clients)) {
                const config = yield this.getClientConfig(key);
                this.joker[key] = yield client.getUser(config.joker);
            }
        });
    }
    /**
     * Authenticate all of the clients in this bot.
     */
    authenticateClients() {
        return __awaiter(this, void 0, void 0, function* () {
            // Await the authentication of the clients linked to the bot.
            for (const client of Object.values(this.clients)) {
                // Await authentication of the bot.
                yield client.authenticate();
            }
        });
    }
    /**
     * Initialize all clients for this bot.
     *
     * Initialization uses the client configuration to properly create the clients.
     */
    initializeClients() {
        return __awaiter(this, void 0, void 0, function* () {
            // Await the processing and initialization of all clients in the configurations.
            for (const clientId of this.config.clients) {
                // Load configuration since it exists.
                const clientConfig = yield this.getClientConfig(ClientType_1.ClientType[clientId]);
                if (Sojiro_1.Sojiro.isEmpty(clientConfig)) {
                    yield Morgana_1.Morgana.warn("Configuration file could not be loaded for the {{client}} client in {{bot}}. This client will not be instantiated." +
                        'To create a configuration file, you can copy the ones found in the "example" bot folder.', {
                        bot: this.id,
                        client: clientId,
                    });
                    return;
                }
                // Uses the ClientFactory to build the appropriate factory given the type.
                // The client is then set to the bot.
                this.clients[ClientType_1.ClientType[clientId]] = yield ClientFactory_1.ClientFactory.buildClient(ClientType_1.ClientType[clientId], clientConfig, this);
            }
        });
    }
}
exports.Bot = Bot;
