"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BotManager = void 0;
// Imports.
const Morgana_1 = require("../Confidant/Morgana");
const Sojiro_1 = require("../Confidant/Sojiro");
const Core_1 = require("../Core/Core");
const Service_1 = require("../Service/Service");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const BotCatalogue_1 = require("./BotCatalogue");
/**
 * Provides a Catalogue for Bots.
 *
 * This class manages the registering and instantiation of bots.
 *
 * Bots are configured in the 'bots' folder at the root of the application.
 *
 */
class BotManager extends Service_1.Service {
    /**
     * Summon a bot that is loaded in the Catalogue.
     *
     * @param id
     *    The ID of the bot to deploy.
     */
    static summon(id) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the bot isn't found, we can't boot it.
            const bot = ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).getBot(id);
            if (!bot) {
                yield Morgana_1.Morgana.warn("Tried to boot an non-existent bot: {{botId}}. Gracefully continuing the program.", { id });
                return;
            }
            // Await deployment handlers for a single bot.
            yield bot.summon();
        });
    }
    /**
     * Shutdown a bot.
     *
     * @param id
     *   ID of the Bot to shutdown.
     */
    static shutdown(id) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the bot isn't found, we can't shut it down.
            const bot = yield ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).getBot(id);
            if (!bot) {
                yield Morgana_1.Morgana.warn("Tried to shutdown an non-existent bot: {{botId}}. Gracefully continuing the program.", { id });
                return;
            }
            yield bot.shutdown();
        });
    }
    /**
     * Boots all bots set up in the 'autoboot' array of the settings.
     */
    static summonAutoBoots() {
        return __awaiter(this, void 0, void 0, function* () {
            // If the autoboot array is empty, we don't do anything here.
            if (Sojiro_1.Sojiro.isEmpty(Core_1.Core.settings.config.bots.autoboot)) {
                yield Morgana_1.Morgana.warn("No bots set up for autobooting. Continuing!");
                return;
            }
            // Boot all bots set up in autobooting.
            for (const id of Core_1.Core.settings.config.bots.autoboot) {
                yield BotManager.summon(id);
                yield Morgana_1.Morgana.success("Successfully Auto-Booted {{bot}}!", { bot: id });
            }
        });
    }
    /**
     * Synethesis handler for the BotManager.
     *
     * Registers all bots and fires all of *their* synthesis handlers.
     */
    synthesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // We'll run build handlers for all bots as this should only be done once.
            // Await preparation handlers for all bots.
            yield Promise.all(ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all().map((bot) => __awaiter(this, void 0, void 0, function* () {
                // Await preparation handler for a single bot.
                yield bot.synthesis();
            })));
        });
    }
    /**
     * Statis handler for the BotManager.
     *
     * Summon the master bot and any bots automatically set for summoning.
     *
     * @inheritDoc
     */
    statis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Boot master bot that will manage all other bots in the codebase.
            yield BotManager.summon(Core_1.Core.settings.config.bots.master);
            // Some more flavor.
            yield Morgana_1.Morgana.success("Booted the master bot, {{bot}}!", { bot: Core_1.Core.settings.config.bots.master });
            // Boot auto-boot bots.
            // Some bots are set up for auto-booting. We'll handle those too.
            yield BotManager.summonAutoBoots();
        });
    }
}
exports.BotManager = BotManager;
