"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientFactory = void 0;
const ClientType_1 = require("./ClientType");
const DiscordClient_1 = require("./Discord/DiscordClient");
const DiscordPrompt_1 = require("./Discord/DiscordPrompt");
const DiscordResonance_1 = require("./Discord/DiscordResonance");
const TwitchClient_1 = require("./Twitch/TwitchClient");
const TwitchPrompt_1 = require("./Twitch/TwitchPrompt");
const TwitchResonance_1 = require("./Twitch/TwitchResonance");
/**
 * Provide a factory class that manages the creation of the right client given a type.
 */
class ClientFactory {
    /**
     * Creates a client instance given a type, bot and configuration.
     *
     * A client is created for each Bot.
     *
     * Each type of client has a different class. We will properly decouple and manage the functionality of each type of
     * client.
     *
     * @param type
     *   Type of client to build.
     * @param config
     *   Configuration object to create the client with, fetched from the bot's configuration file.
     * @param bot
     *   Bot that this client will be linked to.
     *
     * @returns
     *   Client that was instantiated.
     */
    static buildClient(type, config, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the object.
            let client;
            // Depending on the requested type, we build the appropriate client.
            switch (type) {
                // Create a DiscordClient if the type is Discord.
                case ClientType_1.ClientType.Discord: {
                    client = new DiscordClient_1.DiscordClient(bot, config);
                    break;
                }
                // Create a TwitchClient if the type is Discord.
                case ClientType_1.ClientType.Twitch: {
                    client = new TwitchClient_1.TwitchClient(bot, config);
                }
            }
            // Bridge a connection to the application for the client.
            yield client.bridge();
            // Run build tasks for client.
            yield client.build();
            // Return the client.
            return client;
        });
    }
    /**
     * Build a Resonance to send to this client's bot.
     *
     * Creates an object that stores information about a message overheard in the client.
     *
     * @param client
     *   Client to build the resonance for.
     * @param message
     *   The message object obtained from the client.
     *
     * @return
     *   A resonance object containing important information about the message heard.
     */
    static buildResonance(client, message) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the object.
            let resonance;
            // Depending on the requested type, we build the appropriate client.
            switch (client.type) {
                // Create a DiscordClient if the type is Discord.
                case ClientType_1.ClientType.Discord: {
                    resonance = new DiscordResonance_1.DiscordResonance(message.content, message, client.bot, client);
                    break;
                }
                // Create a TwitchClient if the type is Discord.
                case ClientType_1.ClientType.Twitch: {
                    resonance = new TwitchResonance_1.TwitchResonance(message.content, message, client.bot, client);
                }
            }
            // Run build tasks on the resonance.
            yield resonance.build();
            // Return the resonance.
            return resonance;
        });
    }
    /**
     * Build and return a Prompt Object specific to a client.
     *
     * @param promptInfo
     *   Info used to build the prompt object.
     *
     * @returns
     *   A prompt that will be active and set to this client's bot.
     */
    static buildPrompt(promptInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the object.
            let prompt;
            // Depending on the requested type, we build the appropriate client.
            switch (promptInfo.clientType) {
                // Create a DiscordClient if the type is Discord.
                case ClientType_1.ClientType.Discord: {
                    prompt = new DiscordPrompt_1.DiscordPrompt(promptInfo);
                    break;
                }
                // Create a TwitchClient if the type is Discord.
                case ClientType_1.ClientType.Twitch: {
                    prompt = new TwitchPrompt_1.TwitchPrompt(promptInfo);
                }
            }
            // Return the prompt.
            return prompt;
        });
    }
}
exports.ClientFactory = ClientFactory;
