"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiscordClient = void 0;
// Modules
const discord_js_1 = require("discord.js");
const Igor_1 = require("../../Confidant/Igor");
const Morgana_1 = require("../../Confidant/Morgana");
const Sojiro_1 = require("../../Confidant/Sojiro");
const Core_1 = require("../../Core/Core");
const EventSubscriberManager_1 = require("../../Service/EventSubscriber/EventSubscriberManager");
const Client_1 = require("../Client");
const ClientType_1 = require("../ClientType");
/**
 * Provides a class for Discord Clients managed in Lavenza.
 *
 * This class extends hydrabolt's wonderful discord.js package. Couldn't do this
 * without em. Much love!
 *
 * @see https://discord.js.org/#/
 */
class DiscordClient extends Client_1.Client {
    /**
     * @inheritDoc
     */
    constructor(bot, config) {
        super(bot, config);
        /**
         * The type of client.
         *
         * @inheritDoc
         */
        this.type = ClientType_1.ClientType.Discord;
    }
    /**
     * Bridge a connection to Discord.
     *
     * We'll use Discord.JS here! Thank you hydrabolt. :)
     *
     * @inheritDoc
     */
    bridge() {
        return __awaiter(this, void 0, void 0, function* () {
            this.connector = new discord_js_1.Client();
        });
    }
    /**
     * Run build tasks to prepare any necessary functions.
     *
     * Treat this as a proper constructor.
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            // Event: When the client connects to Discord and is ready.
            this.connector.on("ready", () => __awaiter(this, void 0, void 0, function* () {
                // Set game text.
                this.connector.user.setActivity(this.config.activity).catch(console.error);
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "ready", {});
            }));
            // Event: When the discord client connects to a new guild.
            this.connector.on("guildCreate", (guild) => __awaiter(this, void 0, void 0, function* () {
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "guildCreate", { guild });
            }));
            // Event: When the discord client receives a message.
            this.connector.on("message", (message) => __awaiter(this, void 0, void 0, function* () {
                // Ignore messages from any discord bots.
                if (message.author.bot) {
                    return;
                }
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "message", { message });
            }));
            // Event: When the clients disconnects from Discord.
            this.connector.on("disconnected", () => __awaiter(this, void 0, void 0, function* () {
                yield Morgana_1.Morgana.status("Discord client for {{bot}} has disconnected.", { bot: this.bot.id });
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "disconnected");
            }));
            // Event: When the clients disconnects from Discord.
            this.connector.on("error", () => __awaiter(this, void 0, void 0, function* () {
                yield Morgana_1.Morgana.error("Error has occurred for {{bot}}'s client...", { bot: this.bot.id });
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "error");
            }));
        });
    }
    /**
     * Authenticate the client. (Login to Discord)
     *
     * @inheritDoc
     */
    authenticate() {
        return __awaiter(this, void 0, void 0, function* () {
            // Get the token.
            const token = this.bot.env.DISCORD_TOKEN;
            // If the token isn't found, we throw an error.
            if (token === undefined) {
                yield Igor_1.Igor.throw("Discord application token is missing for {{bot}}. Make sure the token is set in the .env file in the bot's folder. See the example bot folder for more details.", { bot: this.bot.id });
            }
            // Await the login in of this client.
            yield this.connector.login(token)
                .catch(() => __awaiter(this, void 0, void 0, function* () {
                yield Igor_1.Igor.throw("Failed to authenticate Discord client for {{bot}}.", { bot: this.bot.id });
            }));
        });
    }
    /**
     * Disconnect from Discord.
     *
     * @inheritDoc
     */
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            // Call the destruction function to disconnect the client.
            yield this.connector.destroy();
            yield Morgana_1.Morgana.warn("Discord client disconnected for {{bot}}.", { bot: this.bot.id });
        });
    }
    /**
     * Bootstrap database folders for Discord client.
     *
     * @inheritDoc
     */
    gestalt() {
        return __awaiter(this, void 0, void 0, function* () {
            // Guild Configurations Sync.
            const guilds = yield Core_1.Core.gestalt().sync({}, `/bots/${this.bot.id}/clients/${this.type}/guilds`);
            // Set up initial guild configurations.
            yield Promise.all(this.connector.guilds.map((guild) => __awaiter(this, void 0, void 0, function* () {
                const baseGuildConfig = {
                    commandPrefix: yield this.bot.config.commandPrefix,
                    name: guild.name,
                    userEminences: {},
                };
                if (!(guild.id in guilds)) {
                    guilds[guild.id] = baseGuildConfig;
                }
                yield Core_1.Core.gestalt().update(`/bots/${this.bot.id}/clients/${this.type}/guilds`, guilds);
            })));
            // Initialize i18n contexts, creating them if they don't exist.
            // Translations are manageable through all of these contexts.
            yield Core_1.Core.gestalt().sync({}, `/i18n/${this.bot.id}/clients/${this.type}/guilds`);
            yield Core_1.Core.gestalt().sync({}, `/i18n/${this.bot.id}/clients/${this.type}/channels`);
            yield Core_1.Core.gestalt().sync({}, `/i18n/${this.bot.id}/clients/${this.type}/users`);
        });
    }
    /**
     * Fetch a Discord user's information.
     *
     * @inheritDoc
     */
    getUser(identifier) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.connector.fetchUser(identifier);
        });
    }
    /**
     * A little utility function to order the bot to type for a set amount of seconds in a given channel.
     *
     * @TODO - Do something about that dumb 'method can be static' message.
     *
     * @param seconds
     *   Amount of seconds to type for.
     * @param channel
     *   The Discord channel to type in.
     */
    typeFor(seconds, channel) {
        return __awaiter(this, void 0, void 0, function* () {
            yield channel.stopTyping();
            yield channel.startTyping(1);
            yield Sojiro_1.Sojiro.wait(seconds);
            yield channel.stopTyping();
        });
    }
    // tslint:disable-next-line:comment-format
    // noinspection JSUnusedGlobalSymbols
    /**
     * Send a cute error message to a destination.
     *
     * @param destination
     *   Destination, normally a channel or a user.
     * @param text
     *   Message of the error.
     * @param type
     *   Type of error. Can be warning, status or error.
     * @param code
     *   Error code.
     *
     * @returns
     *   The message that was sent as an error.
     */
    sendError(destination, { text = "", type = "", code = 404, } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize some variables.
            let message = "";
            let color = "";
            let image;
            // Determine code.
            switch (code) {
                case 401:
                    message = "Unauthorized.";
                    break;
                default:
                    message = "An error has occurred.";
            }
            // Determine color.
            switch (type) {
                case "warning":
                    color = "0xf4d742";
                    image = {
                        attachment: new discord_js_1.Attachment("./core/assets/warning.png", "warning.png"),
                        name: "warning.png",
                    };
                    break;
                default:
                    color = "0xa5201d";
                    image = {
                        attachment: new discord_js_1.Attachment("./core/assets/error.png", "error.png"),
                        name: "error.png",
                    };
            }
            // Send the embed.
            return yield this.sendEmbed(destination, {
                attachments: [
                    image.attachment,
                ],
                color,
                description: text,
                header: {
                    icon: `attachment://${image.name}`,
                    text: `${code}: ${message}`,
                },
                timestamp: true,
            });
        });
    }
    /**
     * Send a embed to a channel.
     *
     * @TODO - We really gotta define an interface for the embed object LOL.
     *
     * @see https://leovoel.github.io/embed-visualizer/
     *
     * @param destination
     *   The destination can be a channel or a user.
     * @param title
     *   The title of the rich embed.
     * @param description
     *   The description of the rich embed.
     * @param header
     *   Object that should contain the *text* of the header and the *icon* if applicable.
     * @param url
     *   URL of the rich embed.
     * @param color
     *   Color of the left border of the rich embed.
     * @param image
     *   Main image of the rich embed. Usually an attachment reference.
     * @param thumbnail
     *   Main thumbnail of the rich embed. Usually an attachment reference.
     * @param fields
     *   Fields of the rich embed.
     * @param footer
     *   Object that should contain the *text* of the footer and the *icon* if applicable.
     * @param attachments
     *   Array of attachments to attach to the embed.
     * @param timestamp
     *   Controls whether or not we want to add a timestamp of the current time.
     *
     * @returns
     *   The message that was sent as an embed.
     */
    sendEmbed(destination, { title = "", description = "", header = {}, url = "", color = "", image = "", thumbnail = "", fields = [], footer = {}, attachments = [], timestamp = false, } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            // Create the embed instance.
            const embed = new discord_js_1.RichEmbed();
            // Manage default values.
            const colorHex = color || "0x1C1CF0";
            // Set default values.
            embed.setColor(colorHex);
            // If timestamp is set to true, set it.
            if (timestamp) {
                embed.setTimestamp(new Date());
            }
            // Set Title if any.
            if (title) {
                embed.setTitle(title);
            }
            // Set Description if any.
            if (description) {
                embed.setDescription(description);
            }
            // Set Header/Author if any.
            if (header) {
                embed.setAuthor(header.text, header.icon);
            }
            // Set Footer if any.
            if (footer) {
                embed.setFooter(footer.text, footer.icon);
            }
            // Set Thumbnail if any.
            if (thumbnail) {
                embed.setThumbnail(thumbnail);
            }
            // Set Image if any.
            if (image) {
                embed.setImage(image);
            }
            // Set fields, if any.
            if (fields) {
                fields.every((field) => {
                    embed.addField(field.name, field.text);
                    return true;
                });
            }
            // Set attachments, if any.
            if (attachments) {
                embed.attachFiles(attachments);
            }
            // If url is set, we set it here.
            if (url) {
                embed.setURL(url);
            }
            // Send the embed.
            return destination.send(embed);
        });
    }
}
exports.DiscordClient = DiscordClient;
