"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TwitchClient = void 0;
// Modules.
const tmi = require("tmi.js");
const Morgana_1 = require("../../Confidant/Morgana");
const Sojiro_1 = require("../../Confidant/Sojiro");
const Core_1 = require("../../Core/Core");
const EventSubscriberManager_1 = require("../../Service/EventSubscriber/EventSubscriberManager");
const Client_1 = require("../Client");
const ClientType_1 = require("../ClientType");
const TwitchChannel_1 = require("./Entity/TwitchChannel");
const TwitchMessage_1 = require("./Entity/TwitchMessage");
const TwitchUser_1 = require("./Entity/TwitchUser");
/**
 * Provides a class for Twitch Clients managed in Lavenza.
 *
 * This class extends tmi.js. Much love to their work and developers!
 *
 * @see https://www.npmjs.com/package/twitch
 */
class TwitchClient extends Client_1.Client {
    /**
     * @inheritDoc
     */
    constructor(bot, config) {
        super(bot, config);
        /**
         * The type of client.
         *
         * @inheritDoc
         */
        this.type = ClientType_1.ClientType.Twitch;
    }
    /**
     * Bridge a connection to Twitch.
     *
     * We'll use TMI here!
     *
     * @inheritDoc
     */
    bridge() {
        return __awaiter(this, void 0, void 0, function* () {
            this.connector = new tmi.Client({
                channels: this.config.channels,
                identity: {
                    password: this.bot.env.TWITCH_OAUTH_TOKEN,
                    username: this.config.username,
                },
            });
        });
    }
    /**
     * Run build tasks to prepare any necessary functions.
     *
     * Treat this as a proper constructor.
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            // Event: When the client connects to Twitch and is ready.
            this.connector.on("connected", () => __awaiter(this, void 0, void 0, function* () {
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "connected", {});
            }));
            // Event: When the twitch client receives a message.
            this.connector.on("message", (target, context, message, self) => __awaiter(this, void 0, void 0, function* () {
                // Ignore messages from this bot.
                if (self) {
                    return;
                }
                // Build the author information.
                const author = new TwitchUser_1.TwitchUser(context.username, context.username, context["display-name"]);
                // Build channel information.
                const channel = new TwitchChannel_1.TwitchChannel(target, target.replace("#", ""), context["message-type"]);
                // Build the message.
                const msg = new TwitchMessage_1.TwitchMessage(message, author, channel, context);
                // Run Event Subscribers for this event.
                yield EventSubscriberManager_1.EventSubscriberManager.runEventSubscribers(this, "message", { msg });
            }));
        });
    }
    /**
     * Authenticate the client. (Connect to Twitch)
     */
    authenticate() {
        return __awaiter(this, void 0, void 0, function* () {
            // Simply call TMI's connect function.
            yield this.connector.connect();
        });
    }
    /**
     * Disconnect from Twitch.
     */
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            // Simply call TMI's disconnect function.
            yield this.connector.disconnect();
            yield Morgana_1.Morgana.warn("Twitch client disconnected for {{bot}}.", { bot: this.bot.id });
        });
    }
    /**
     * Bootstrap database folders for Twitch client.
     *
     * @inheritDoc
     */
    gestalt() {
        return __awaiter(this, void 0, void 0, function* () {
            // We sync the client configurations.
            const channels = yield Core_1.Core.gestalt().sync({}, `/bots/${this.bot.id}/clients/${this.type}/channels`);
            // Generate configuration for each channel.
            yield Promise.all(this.config.channels.map((channel) => __awaiter(this, void 0, void 0, function* () {
                // For all guilds, we initialize this default configuration.
                const baseChannelConfig = {
                    commandPrefix: this.bot.config.commandPrefix,
                    name: channel,
                    userEminences: {},
                };
                if (!(channel in channels)) {
                    channels[channel] = baseChannelConfig;
                }
                yield Core_1.Core.gestalt().update(`/bots/${this.bot.id}/clients/${this.type}/channels`, channels);
            })));
            // Initialize i18n contexts, creating them if they don't exist.
            // Translations are manageable through all of these contexts.
            yield Core_1.Core.gestalt().sync({}, `/i18n/${this.bot.id}/clients/${this.type}/channels`);
            yield Core_1.Core.gestalt().sync({}, `/i18n/${this.bot.id}/clients/${this.type}/users`);
        });
    }
    /**
     * Fetch a Twitch user's information.
     *
     * @inheritDoc
     */
    getUser(identifier) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                displayName: identifier,
                id: identifier,
                username: identifier,
            };
        });
    }
    /**
     * A little utility function to order the bot to type for a set amount of seconds in a given channel.
     *
     * @param seconds
     *   Amount of seconds to type for.
     * @param channel
     *   The Twitch channel to type in. Not necessary since it doesn't change anything.
     */
    typeFor(seconds, channel) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Sojiro_1.Sojiro.wait(seconds);
        });
    }
}
exports.TwitchClient = TwitchClient;
