"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 *
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Core = void 0;
// Modules.
const colors = require("colors");
const path = require("path");
const prompts = require("prompts");
// Imports.
const PackageInfo = require("../../../../package.json");
const BotManager_1 = require("../Bot/BotManager");
const Akechi_1 = require("../Confidant/Akechi");
const Morgana_1 = require("../Confidant/Morgana");
const Sojiro_1 = require("../Confidant/Sojiro");
const Yoshida_1 = require("../Confidant/Yoshida");
const Gestalt_1 = require("../Gestalt/Gestalt");
const RuntimeProcessId_1 = require("../Service/RuntimeProcessId");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const TalentManager_1 = require("../Talent/TalentManager");
const CoreStatus_1 = require("./CoreStatus");
// Configure colors for console.
// Set console color themes.
/** @see https://www.npmjs.com/package/colors */
colors.setTheme({
    data: "grey",
    debug: "blue",
    error: "red",
    help: "cyan",
    input: "grey",
    prompt: "grey",
    silly: "rainbow",
    status: "blue",
    success: "cyan",
    verbose: "cyan",
    warning: "yellow",
});
/**
 * Provides class for the Core of the Lavenza application.
 *
 * Most of the Core business and bootstrapping happens here, though specified features will be
 * properly divided into respective classes. We're going for full-blown OOP here.
 * Let's try to make, and KEEP, this clean now!
 *
 * Lavenza hates dirty code. ;)
 */
class Core {
    /**
     * Initialize Lavenza's configurations with the specified path.
     *
     * With the provided path, we will tell the rest of the code where to look for important files. Projects that
     * inherit Lavenza will need to create relevant files that are needed to use the framework.
     *
     * By default, we get the path to the entrypoint script of the module that called Lavenza.
     */
    static initialize(root = path.dirname(require.main.filename)) {
        return __awaiter(this, void 0, void 0, function* () {
            // Some flavor text for the console.
            yield Morgana_1.Morgana.success("-----------------------------------------------");
            yield Morgana_1.Morgana.success(`INITIALIZING LAVENZA (v${Core.version})...`);
            yield Morgana_1.Morgana.success("-----------------------------------------------");
            // Path to the Lavenzafile.
            const pathToLavenzafile = `${root}/.lavenza.yml`;
            // Variable to store the path to the framework installation in the module that is calling Lavenza.
            let pathToLavenzaInstallation;
            // Check if the .lavenza.yml file exists at this path.
            if (yield Akechi_1.Akechi.fileExists(`${root}/.lavenza.yml`)) {
                // Set settings values to the class.
                Core.settings = (yield Akechi_1.Akechi.readYamlFile(pathToLavenzafile));
                pathToLavenzaInstallation = `${root}/${Core.settings.root}`;
            }
            else {
                yield Morgana_1.Morgana.warn(`A Lavenzafile could not be located at "${root}". Use the "lavenza init" command to initialize the base config file and set this up.`);
                process.exit(1);
            }
            // So first, we want to check if the provided path exists.
            // We're already going to start making use of our Confidants here! Woo!
            if (!Akechi_1.Akechi.isDirectory(pathToLavenzaInstallation)) {
                yield Morgana_1.Morgana.warn(`The path configured (${pathToLavenzaInstallation}) doesn't lead to a valid Lavenza installation directory.`);
                // Start a prompt to see if user would like to generate a basic Desk.
                yield Morgana_1.Morgana.warn("Would you like to generate an installation at this path?");
                const { confirmation } = yield prompts({
                    initial: true,
                    message: "Yes or no (Y/N) ?",
                    name: "confirmation",
                    type: "confirm",
                });
                // If they agree, copy the basic desk to their desired location.
                if (confirmation) {
                    yield Akechi_1.Akechi.copyFiles(path.resolve(__dirname, "../../../templates/installation"), pathToLavenzaInstallation);
                    yield Morgana_1.Morgana.success("An installation has been generated at the provided path. You may configure it and try running Lavenza again!");
                }
                yield Morgana_1.Morgana.error("Until an installation is properly configured, Lavenza will not run properly. Please refer to the guides in the README to configure Lavenza.");
                process.exit(1);
            }
            // Using the provided path, we set the relevant paths to the Core.
            yield Core.setPaths(pathToLavenzaInstallation);
            // Initialize Yoshida's translation options since we'll be using them throughout the application.
            yield Yoshida_1.Yoshida.initializeI18N();
            // If a master isn't set, we shouldn't continue. A master bot must set.
            if (Sojiro_1.Sojiro.isEmpty(Core.settings.config.bots.master)) {
                yield Morgana_1.Morgana.error("There is no master bot set in your Lavenzafile. A master bot must be set so the application knows which bot manages everything!");
                process.exit(1);
            }
            // If a master bot is set but doesn't exist, we shouldn't continue either.
            if (!(yield Akechi_1.Akechi.directoryExists(`${Core.paths.bots}/${Core.settings.config.bots.master}`))) {
                yield Morgana_1.Morgana.error(`The configured master bot, "${Core.settings.config.bots.master}", does not exist. Run "lavenza provision" or "lavenza generate bot ${Core.settings.config.bots.master}" to fix this.`);
                yield Morgana_1.Morgana.error("It is highly recommended to run \"lavenza provision\" to make sure that everything is configured correctly!");
                process.exit(1);
            }
            // Now we'll perform core service loading.
            yield ServiceContainer_1.ServiceContainer.load(path.resolve(__dirname, "../core.services.yml"));
            // Return the core so we can chain functions.
            return Core;
        });
    }
    /**
     * PERSONA!
     *
     * This function goes through the 4 main phases of execution in the application:
     *  - Genesis: The creation of base core functionalities and classes. Very primordial setup steps are in this phase.
     *  - Synthesis: Linking of various parts of the application.
     *  - Statis: Stabilization and database connections and initializations.
     *  - Symbiosis: The final stage where everything runs in harmony!
     *
     * Services are used to manage all of these steps and isolate functionality intotheir own environments.
     *
     * Refer to "core.services.yml" for a list of services declared in Lavenza, as well as their runtime order.
     */
    static summon() {
        return __awaiter(this, void 0, void 0, function* () {
            // If settings aren't set, it means that initialization was bypassed. We can't allow that.
            if (Sojiro_1.Sojiro.isEmpty(Core.settings)) {
                return;
            }
            // Set the core status to "GENESIS" & perform genesis tasks for all services.
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            yield Morgana_1.Morgana.warn("[-- PHASE 1/4: GENESIS --]");
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            Core.status = CoreStatus_1.CoreStatus.genesis;
            yield ServiceContainer_1.ServiceContainer.tasks(RuntimeProcessId_1.RuntimeProcessId.genesis);
            // Set the core status to "SYNTHESIS" & perform synthesis tasks for all services
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            yield Morgana_1.Morgana.warn("[-- PHASE 2/4: SYNTHESIS --]");
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            Core.status = CoreStatus_1.CoreStatus.synthesis;
            yield ServiceContainer_1.ServiceContainer.tasks(RuntimeProcessId_1.RuntimeProcessId.synthesis);
            // Set the core status to "STATIS" & perform statis tasks for all services.
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            yield Morgana_1.Morgana.warn("[-- PHASE 3/4: STATIS --]");
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            Core.status = CoreStatus_1.CoreStatus.statis;
            yield ServiceContainer_1.ServiceContainer.tasks(RuntimeProcessId_1.RuntimeProcessId.statis);
            // Set the core status to "SYMBIOSYS" & perform symbiosis tasks for all services.
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            yield Morgana_1.Morgana.warn("[-- PHASE 4/4: SYMBIOSIS --]");
            yield Morgana_1.Morgana.warn("-----------------------------------------------");
            Core.status = CoreStatus_1.CoreStatus.symbiosis;
            yield ServiceContainer_1.ServiceContainer.tasks(RuntimeProcessId_1.RuntimeProcessId.symbiosis);
            // Set the core status to "RUNNING".
            Core.status = CoreStatus_1.CoreStatus.running;
            // Some more flavor text.
            yield Morgana_1.Morgana.success("-----------------------------------------------");
            yield Morgana_1.Morgana.wonderful(`██╗    ██╗ ██████╗ ███╗   ██╗██████╗ ███████╗██████╗ ███████╗██╗   ██╗██╗     ██╗
██║    ██║██╔═══██╗████╗  ██║██╔══██╗██╔════╝██╔══██╗██╔════╝██║   ██║██║     ██║
██║ █╗ ██║██║   ██║██╔██╗ ██║██║  ██║█████╗  ██████╔╝█████╗  ██║   ██║██║     ██║
██║███╗██║██║   ██║██║╚██╗██║██║  ██║██╔══╝  ██╔══██╗██╔══╝  ██║   ██║██║     ╚═╝
╚███╔███╔╝╚██████╔╝██║ ╚████║██████╔╝███████╗██║  ██║██║     ╚██████╔╝███████╗██╗
 ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═══╝╚═════╝ ╚══════╝╚═╝  ╚═╝╚═╝      ╚═════╝ ╚══════╝╚═╝`);
            yield Morgana_1.Morgana.success("Lavenza should now be running! Scroll up in the logs to see if any errors occurred and handle them as needed. :)");
        });
    }
    /**
     * Fetch a service from the Service Container.
     */
    static service(id) {
        // Find the service through the Service Container.
        return ServiceContainer_1.ServiceContainer.get(id);
    }
    /**
     * Obtain Gestalt service.
     *
     * @return
     *   The Gestalt databse service.
     */
    static gestalt() {
        return Core.service(Gestalt_1.Gestalt);
    }
    /**
     * Obtain BotManager from Services.
     *
     * @return
     *   The Bot Catalogue.
     */
    static botManager() {
        return Core.service(BotManager_1.BotManager);
    }
    /**
     * Obtain BotManager from Services.
     *
     * @return
     *   The Bot Catalogue.
     */
    static talentManager() {
        return Core.service(TalentManager_1.TalentManager);
    }
    /**
     * Setup paths and assign them to the Core.
     *
     * @param rootPath
     *   The provided root path to base ourselves on.
     */
    static setPaths(rootPath) {
        return __awaiter(this, void 0, void 0, function* () {
            Core.paths = {
                bots: `${rootPath}/bots`,
                database: `${rootPath}/database`,
                root: rootPath,
                talents: {
                    core: path.resolve(__dirname, "../../../talents"),
                    custom: `${rootPath}/talents`,
                },
            };
        });
    }
}
exports.Core = Core;
/**
 * Store the current status of the application.
 */
Core.status = CoreStatus_1.CoreStatus.sleep;
/**
 * Stores Lavenza's version.
 * The version number is obtained from the 'package.json' file at the root of the project.
 */
Core.version = PackageInfo.version;
