"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GestaltComposer = void 0;
const BotCatalogue_1 = require("../Bot/BotCatalogue");
const Sojiro_1 = require("../Confidant/Sojiro");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const TalentCatalogue_1 = require("../Talent/TalentCatalogue");
const Composer_1 = require("./Composer/Composer");
/**
 * Provides a main composer for the Gestalt main service.
 */
class GestaltComposer extends Composer_1.Composer {
    constructor() {
        super(...arguments);
        /**
         * The priority of the Resonator. This determines the order in which Resonators will resonate.
         */
        this.priority = 5000;
    }
    /**
     * Each composer service must implement this function to determine what to do when the Manager calls them.
     *
     * Composers are made to do database bootstrapping during runtime.
     */
    compose() {
        return __awaiter(this, void 0, void 0, function* () {
            // @TODO - Run each service's Gestalt handler.
            // @TODO - We can do the same logic as the Plugin Seekers & Event Subscribers.
            // Creation of i18n collection.
            // All data pertaining to translations will be saved here.
            yield this.gestaltService.createCollection("/i18n");
            // Creation of the Bots collection.
            yield this.gestaltService.createCollection("/bots");
            // Run Gestalt handlers for each Bot.
            for (const bot of yield ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all()) {
                // Initialize the database collection for this bot if it doesn't already exist.
                yield this.gestaltService.createCollection(`/bots/${bot.id}`);
                // Initialize the database collection for this bot's configurations if it doesn't already exist.
                yield this.gestaltService.createCollection(`/bots/${bot.id}/config`);
                // Sync core bot config to the database.
                yield this.gestaltService.sync(bot.config, `/bots/${bot.id}/config/core`);
                // Initialize i18n database collection for this bot if it doesn't already exist.
                yield this.gestaltService.createCollection(`/i18n/${bot.id}`);
                // Initialize i18n database collection for this bot's clients configurations if it doesn't already exist.
                yield this.gestaltService.createCollection(`/i18n/${bot.id}/clients`);
                // Create a database collection for the talents granted to a bot.
                yield this.gestaltService.createCollection(`/bots/${bot.id}/talents`);
                // For all Talents belonging to this bot, we run additional tasks.
                const botTalents = yield ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalentsForBot(bot);
                if (botTalents) {
                    for (const botTalent of botTalents) {
                        // Create a database collection for the talents granted to a Bot.
                        yield this.gestaltService.createCollection(`/bots/${bot.id}/talents/${botTalent.machineName}`);
                        // Await the synchronization of data between the Talent's default configuration and the database configuration.
                        yield this.gestaltService.sync(botTalent.config, `/bots/${bot.id}/talents/${botTalent.machineName}/config`);
                    }
                }
                // Create a database collection for the clients belonging to a Bot.
                yield this.gestaltService.createCollection(`/bots/${bot.id}/clients`);
                // For all Clients belonging to this bot, we run additional tasks.
                for (const client of Object.values(bot.clients)) {
                    // Make sure database collection exists for this client for the given bot.
                    yield this.gestaltService.createCollection(`/bots/${bot.id}/clients/${client.type}`);
                    // Make sure database collection exists for this client's i18n storage for the given bot.
                    yield this.gestaltService.createCollection(`/i18n/${bot.id}/clients/${client.type}`);
                    // Run each client's gestalt handler.
                    yield client.gestalt();
                }
            }
            // Creation of the Talents collection.
            yield this.gestaltService.createCollection("/talents");
            // Run Gestalt handlers for each Talent.
            for (const talent of ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).all()) {
                // Initialize the database collection for this talent if it doesn't already exist.
                yield this.gestaltService.createCollection(`/talents/${talent.machineName}`);
            }
        });
    }
    /**
     * Get the active configuration from the database for this Bot.
     *
     * If no active configuration is found, it is fetched from the base configurations and synced to the database.
     *
     * @param bot
     *   Bot to get configurations for.
     *
     * @returns
     *   Returns the configuration fetched from the database.
     */
    getActiveConfigForBot(bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Attempt to get the active configuration from the database.
            const activeConfig = yield this.gestaltService.get(`/bots/${bot.id}/config/core`);
            if (!Sojiro_1.Sojiro.isEmpty(activeConfig)) {
                return activeConfig;
            }
            // Sync it to the database.
            yield this.gestaltService.sync(bot.config, `/bots/${bot.id}/config/core`);
            // Return the configuration.
            return bot.config;
        });
    }
    /**
     * Retrieve active client configuration for this bot.
     *
     * If no active configuration is found, it is fetched from the base configurations and synced to the database.
     *
     * @param bot
     *   Bot to get configurations for.
     * @param clientType
     *   The type of client configuration to return for the bot.
     *
     * @returns
     *   The requested client configuration straight from the database.
     */
    getActiveClientConfigForBot(bot, clientType) {
        return __awaiter(this, void 0, void 0, function* () {
            // Attempt to get the active configuration from the database.
            const activeConfig = yield this.gestaltService.get(`/bots/${bot.id}/config/${clientType}`);
            if (!activeConfig) {
                return activeConfig;
            }
            // If we don't find any configurations in the database, we'll fetch it normally and then save it.
            const config = yield bot.getClientConfig(clientType);
            // Sync it to the database.
            yield this.gestaltService.sync(config, `/bots/${bot.id}/config/${clientType}`);
            // Return the configuration.
            return config;
        });
    }
    /**
     * Get active configurations for client
     *
     * @param client
     *   The client to get configurations for.
     *
     * @return
     *   The client configurations loaded from the database.
     */
    getActiveConfigForClient(client) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.gestaltService.get(`/bots/${client.bot.id}/clients/${client.type}`);
        });
    }
    /**
     * Get the active configuration from the database for this Talent, in the context of a Bot.
     *
     * Bots can override talent configurations for themselves. As a result, in the database, we must store configurations
     * specific to this talent in the bot's database table.
     *
     * @param talent
     *   The talent we want to fetch configurations for.
     * @param bot
     *   The bot context for the configuration we want to fetch. Each bot can have different configuration overrides
     *   for talents.
     *
     * @returns
     *   The active database configuration for the talent configuration, specific to a given Bot.
     */
    getActiveTalentConfigForBot(talent, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Await Gestalt's API call to get the configuration from the storage.
            return yield this.gestaltService.get(`/bots/${bot.id}/talents/${talent.machineName}/config`);
        });
    }
}
exports.GestaltComposer = GestaltComposer;
