"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resonance = void 0;
const ClientFactory_1 = require("../Client/ClientFactory");
const Igor_1 = require("../Confidant/Igor");
const Sojiro_1 = require("../Confidant/Sojiro");
const Yoshida_1 = require("../Confidant/Yoshida");
/**
 * Provides a model that regroups information about a message received from a client.
 *
 * I love the name. But yes, this is a model to house a collection of useful information about a message a bot
 * receives.
 *
 * To manage different types of clients, this abstract class acts as a parent class to child classes that are more
 * client specific. This being said, child classes make good use of functions and properties in this class.
 */
class Resonance {
    /**
     * Resonance constructor.
     *
     * @param content
     *   The raw string content of the message received. This is deciphered by the bot.
     * @param message
     *   The message object received, unmodified. This is useful, since different clients might send different kinds of
     *   messages. It's always nice to have the original message.
     * @param bot
     *   The bot that heard this resonance.
     * @param client
     *   The client that sent the original message.
     */
    constructor(content, message, bot, client) {
        this.content = content;
        this.message = message;
        this.bot = bot;
        this.client = client;
    }
    /**
     * Perform build tasks for a Resonance.
     *
     * This function runs shortly after a resonance is constructed. Consider this an asynchronous constructor.
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            this.origin = yield this.resolveOrigin();
            this.locale = yield this.getLocale();
            this.private = yield this.resolvePrivacy();
        });
    }
    /**
     * Returns whether or not this Resonance was heard in a private channel.
     *
     * @returns
     *   Returns true if the Resonance was heard in a private channel. Returns false otherwise.
     */
    isPrivate() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.private === "public";
        });
    }
    /**
     * Set up a prompt to a specified user.
     *
     * @param promptInfo
     *   Info used to build the prompt object.
     */
    prompt(promptInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            // Set prompt user info tied to this resonance if we must.
            if (!promptInfo.user) {
                promptInfo.user = this.author;
            }
            // Set prompt channel info tied to this resonance if we must.
            if (!promptInfo.channel) {
                promptInfo.channel = this.channel;
            }
            // Set prompt channel info tied to this resonance if we must.
            if (!promptInfo.clientType) {
                promptInfo.clientType = this.client.type;
            }
            // Set prompt bot.
            promptInfo.bot = this.bot;
            // Set resonance.
            promptInfo.resonance = this;
            // Build the prompt.
            const prompt = yield ClientFactory_1.ClientFactory.buildPrompt(promptInfo);
            // Set the prompt to the bot.
            this.bot.prompts.push(prompt);
            // Await resolution of the prompt.
            return prompt.await();
        });
    }
    /**
     * Reply to the current resonance.
     *
     * This will send a formatted reply to the Resonance, replying directly to the message that was heard.
     *
     * @param content
     *   Content to send back.
     * @param personalizationTag
     *   Tag for text personalizations, if needed. This will use Yoshida to get personalizations for the current bot.
     *
     * @returns
     *   We'll receive a Promise containing the message that was sent in the reply, allowing us to
     *   act upon it if needed.
     */
    reply(content, personalizationTag = "") {
        return __awaiter(this, void 0, void 0, function* () {
            // Basically call the send method, but we already know the destination.
            return this.send(this.origin, content, personalizationTag);
        });
    }
    /**
     * Reply to the current resonance, with a translating the string.
     *
     * This will send a formatted reply to the Resonance, replying directly to the message that was heard.
     *
     * We make this function flexible, similar to how i18n.__() works. It will allow multiple definitions for function
     * calls and act depending on what types of parameters are given.
     *
     * @param parameters
     *   Parameters to parse from the call.
     *
     * @returns
     *   We'll receive a Promise containing the message that was sent in the reply, allowing us to
     *   act upon it if needed.
     */
    __reply(...parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            // Parse the parameters obtained.
            const params = yield Yoshida_1.Yoshida.parseI18NParams(parameters);
            // Basically call the send method, but we already know the destination.
            return this.__send(this.origin, { phrase: params.phrase, locale: params.locale, tag: params.tag }, params.replacers, "PARSED")
                .catch(Igor_1.Igor.stop);
        });
    }
    /**
     * Send a message to a destination using information from the resonance.
     *
     * This will send a formatted message to it's destination.
     *
     * This function acts as a shortcut to sending messages back to a resonance.
     *
     * @param destination
     *   Destination to send this message to.
     * @param content
     *   Content to send back to the destination.
     * @param personalizationTag
     *   Tag for text personalizations, if needed. This will use Yoshida to get personalizations for the current bot.
     *
     * @returns
     *   We'll receive a Promise containing the message that was sent in the reply, allowing us to
     *   act upon it if needed. This is useful for Discord.
     */
    send(destination, content, personalizationTag = "") {
        return __awaiter(this, void 0, void 0, function* () {
            // Store variable for any content that is altered.
            let alteredContent = content;
            // If a personalization tag is set, we want to use Yoshida to get a personalization for this bot.
            if (!Sojiro_1.Sojiro.isEmpty(personalizationTag)) {
                alteredContent = yield Yoshida_1.Yoshida.personalize(alteredContent, personalizationTag, this.bot);
            }
            // If all fails, we'll simply use this instance's doSend function.
            // Which will currently crash the program.
            return this.doSend(this.bot, destination, alteredContent)
                .catch(Igor_1.Igor.stop);
        });
    }
    /**
     * Send a message to a destination using information from the resonance, translating the text.
     *
     * This will send a formatted message to it's destination.
     *
     * This function acts as a shortcut to sending messages back to a resonance. It
     * will handle translation of the text as well.
     *
     * We make this function flexible, similar to how i18n.__() works. It will allow multiple definitions for function
     * calls and act depending on what types of parameters are given.
     *
     * @param destination
     *   Destination to send this message to.
     * @param parameters
     *   Parameters to parse from the call.
     *
     * @returns
     *   We'll receive a Promise containing the message that was sent in the reply, allowing us to
     *   act upon it if needed.
     */
    __send(destination, ...parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            // Parse the parameters obtained.
            const params = yield Yoshida_1.Yoshida.parseI18NParams(parameters);
            // If a locale is not set in the parameters, we need to determine what it is using the Resonance.
            if (params.locale === undefined) {
                params.locale = yield this.locale;
            }
            // If a locale is STILL not defined after the above code, we set it to the default one set to the bot.
            if (params.locale === undefined) {
                params.locale = this.bot.config.locale || "en";
            }
            // If a personalization tag is set, we want to use Yoshida to get a personalization for this bot.
            if (!Sojiro_1.Sojiro.isEmpty(params.tag)) {
                params.phrase = yield Yoshida_1.Yoshida.personalize(params.phrase, params.tag, this.bot)
                    .catch(Igor_1.Igor.stop);
            }
            // Now, using the information from the parameters, we fetch necessary translations.
            const content = yield Yoshida_1.Yoshida.translate({ phrase: params.phrase, locale: params.locale }, params.replacers, "PARSED");
            // Invoke the regular send function.
            return this.send(destination, content);
        });
    }
}
exports.Resonance = Resonance;
