"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResonanceEventSubscriber = void 0;
const ClientFactory_1 = require("../Client/ClientFactory");
const ClientType_1 = require("../Client/ClientType");
const Core_1 = require("../Core/Core");
const CoreStatus_1 = require("../Core/CoreStatus");
const EventSubscriber_1 = require("../Service/EventSubscriber/EventSubscriber");
const ServiceContainer_1 = require("../Service/ServiceContainer");
/**
 * Provides an Event Subscriber for the Resonance Service.
 */
class ResonanceEventSubscriber extends EventSubscriber_1.EventSubscriber {
    /**
     * Get the list of subscribed events for given clients.
     */
    getEventSubcriptions() {
        return {
            [ClientType_1.ClientType.Discord]: {
                // When the "message" event occurs, fire the "resonate" function with a priority of "5000".
                message: { method: "resonate", priority: 5000 },
            },
            [ClientType_1.ClientType.Twitch]: {
                // When the "message" event occurs, fire the "resonate" function with a priority of "5000".
                message: { method: "resonate", priority: 5000 },
            },
        };
    }
    /**
     * Resonate a message heard in a client.
     *
     * Now, explanations.
     *
     * This function will be used to send a 'communication' back to the bot. This happens whenever a message
     * is 'heard', meaning that the bot is in a chat room and a message was sent by someone (or another bot).
     *
     * When this function is ran, we fetch the raw content of the message sent, and we build a Resonance object with it.
     * This is a fancy name for an object that stores information about a received communication.
     *
     * We then invoke all Resonator services that are declared for the given bot. Resonators are services that other
     * parts of the core or talents can provide to act upon messages that are heard.
     *
     * @param client
     *   The client that emitted the event.
     * @param message
     *   The message that was passed through the event.
     */
    // tslint:disable-next-line:completed-docs
    resonate(client, { message }) {
        return __awaiter(this, void 0, void 0, function* () {
            // We'll only do the following tasks if the Core is in the proper status.
            // Resonators will only run if the Lavenza Core is completely finished preparations.
            if (Core_1.Core.status !== CoreStatus_1.CoreStatus.running) {
                return;
            }
            // Construct a 'Resonance'.
            const resonance = yield ClientFactory_1.ClientFactory.buildResonance(client, message);
            // Obtain all resonator services.
            const resonators = ServiceContainer_1.ServiceContainer.getServicesWithTag("resonator");
            // Sort resonators in order of defined priority.
            resonators.sort((a, b) => b.priority - a.priority);
            // Run them all.
            for (const resonator of resonators) {
                // First, we check if the service defined is a service that is available for the bot in the client.
                if (resonator.talent && !client.bot.config.talents.includes(resonator.talent)) {
                    continue;
                }
                // For each resonator, we run the resonate() function.
                yield resonator.resonate(resonance);
            }
        });
    }
}
exports.ResonanceEventSubscriber = ResonanceEventSubscriber;
