"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Catalogue = void 0;
// Modules.
const lodash = require("lodash");
// Imports.
const Sojiro_1 = require("../../Confidant/Sojiro");
const Service_1 = require("../Service");
/**
 * Provides a base class for Catalogues.
 *
 * Catalogues are classes that handle storage & retrieval of specified objects.
 */
class Catalogue extends Service_1.Service {
    constructor() {
        super(...arguments);
        /**
         * Repository for this catalogue.
         *
         * The repository is an array containing the full list of objects that the catalogue will store.
         */
        this.repository = [];
        /**
         * Libraries for this catalogue.
         *
         * This will store different 'libraries' for this catalogue, adding the ability to organize objects into different
         * spaces.
         *
         * A concrete example would be a FoodCatalogue. You store Food items in it. When you store Food items, you can tag
         * it with different Libraries to store it in, such as 'fruits' if the Food you're storing is a Fruit.
         */
        this.libraries = new Map();
    }
    /**
     * Retrieve a full list of all items, regardless of storage.
     */
    all() {
        return this.repository;
    }
    /**
     * Retrieve a library of the catalogue given a unique library ID.
     *
     * @param id
     *   The id of the library to retrieve.
     */
    library(id) {
        if (!this.libraries.get(id)) {
            return [];
        }
        return this.libraries.get(id);
    }
    /**
     * Retrieve one of the items with a provided predicate.
     *
     * @param predicate
     *   Function to use to filter the items we want to retrieve.
     * @param library
     *   The library to search in, if specified.
     */
    find(predicate, library) {
        // If the library is specified, we'd like to fetch only from the subset of objects that are within that library.
        if (library) {
            if (!this.library(library)) {
                return;
            }
            return this.library(library).find(predicate);
        }
        // Otherwise, search in the full repository.
        return this.repository.find(predicate);
    }
    /**
     * Retrieve many items given the provided predicate.
     *
     * @param predicate
     *   Function to use to filter the items we want to retrieve.
     * @param library
     *   The library to retrieve from, if specified.
     */
    retrieve(predicate, library) {
        // If the section is specified, we'd like to fetch only from the subset of objects that are within that section.
        if (library) {
            if (!this.library(library)) {
                return;
            }
            return this.library(library).filter(predicate);
        }
        // If no library is specified, we simply filter through the full repository.
        return this.repository.filter(predicate);
    }
    /**
     * Add an element to a section in the catalogue.
     *
     * @param payload
     *   The item(s) to add.
     * @param library
     *   Section to update if need be.
     */
    assign(payload, library) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the payload is not an array, we put it in one.
            if (!(payload instanceof Array)) {
                payload = [payload];
            }
            // First, we push the object into the repository.
            this.repository = [...this.repository, ...payload];
            // Unique values.
            this.repository = lodash.uniq(this.repository);
            // We'll add the payload to the appropriate library.
            const libObjects = this.libraries.get(library) || [];
            let objects = [...libObjects, ...payload];
            objects = lodash.uniq(objects);
            this.libraries.set(library, objects);
        });
    }
    /**
     * Remove an item from a section in the catalogue.
     *
     * @param item
     *   Item to remove.
     * @param library
     *   Library to remove from.
     */
    unassign(item, library) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the library doesn't exist, or is empty, we should just exit.
            if (!this.library(library) || (yield Sojiro_1.Sojiro.isEmpty(this.library(library)))) {
                return;
            }
            const currentObjectList = this.library(library);
            yield Sojiro_1.Sojiro.removeFromArray(currentObjectList, item);
        });
    }
    /**
     * Add an element into a storage.
     *
     * @param payload
     *   The item(s) to add.
     * @param library
     *   Library to add items in.
     */
    store(payload, library) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the payload is not an array, we put it in one.
            if (!(payload instanceof Array)) {
                payload = [payload];
            }
            // First, we push the object into the repository.
            this.repository = [...this.repository, ...payload];
            // Unique values.
            this.repository = lodash.uniq(this.repository);
            // If a library is specified, we'll add the payload to the appropriate one.
            if (library) {
                const libObjects = this.libraries.get(library) || [];
                let objects = [...libObjects, ...payload];
                objects = lodash.uniq(objects);
                this.libraries.set(library, objects);
            }
        });
    }
    /**
     * Remove an item from the catalogue.
     *
     * @param item
     *   Item to remove from the storage
     * @param library
     *   Library to remove from.
     */
    pop(item, library) {
        return __awaiter(this, void 0, void 0, function* () {
            // Remove the item from the repository itself and all sections.
            yield Sojiro_1.Sojiro.removeFromArray(this.repository, item);
            this.libraries.forEach((lib) => __awaiter(this, void 0, void 0, function* () { return Sojiro_1.Sojiro.removeFromArray(lib, item); }));
        });
    }
}
exports.Catalogue = Catalogue;
