"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventSubscriberManager = void 0;
const Service_1 = require("../Service");
const ServiceContainer_1 = require("../ServiceContainer");
const SubscriptionRecordCatalogue_1 = require("./SubscriptionRecordCatalogue");
/**
 * Provides a manager for event subscribers.
 */
class EventSubscriberManager extends Service_1.Service {
    /**
     * Run event subscribers for a given client & event.
     *
     * @param client
     *   Client the run event subscribers for.
     * @param event
     *   Event to handle.
     * @param data
     *   Data obtained from the event.
     */
    static runEventSubscribers(client, event, data) {
        return __awaiter(this, void 0, void 0, function* () {
            // Through the catalogue, we get the array of subscription records that are stored in the appropriate library.
            const records = ServiceContainer_1.ServiceContainer.get(SubscriptionRecordCatalogue_1.SubscriptionRecordCatalogue).library(`${client.type}::${event}`);
            // If there are no records, we can exit here.
            if (!records) {
                return;
            }
            // We want to sort each record by priority real fast.
            records.sort((a, b) => b.priority - a.priority);
            // Loop through all the records and run those that we need to run.
            for (const record of records) {
                // First, we check if the service defined in the record is a service that is available for the bot in the client.
                if (record.service.talent && !client.bot.config.talents.includes(record.service.talent)) {
                    continue;
                }
                // Now we simply run the subscriber.
                yield record.service[record.method](client, data);
            }
        });
    }
    /**
     * Synthesize event subscriber services.
     *
     * This will assign and subscribe all EventSubscribers to their customized events.
     */
    static synthesizeEventSubscribers() {
        return __awaiter(this, void 0, void 0, function* () {
            // Now we want to obtain all Event Subscriber services and populate the subscription catalogue.
            for (const service of ServiceContainer_1.ServiceContainer.getServicesWithTag("event_subscriber")) {
                const subscriptions = service.getEventSubcriptions();
                for (const [client, events] of Object.entries(subscriptions)) {
                    for (const [event, subscription] of Object.entries(events)) {
                        yield ServiceContainer_1.ServiceContainer.get(SubscriptionRecordCatalogue_1.SubscriptionRecordCatalogue)
                            .store({
                            event,
                            method: subscription.method,
                            priority: subscription.priority,
                            service,
                        }, `${client}::${event}`);
                    }
                }
            }
        });
    }
    /**
     * Run Synthesis process handler.
     *
     * @inheritDoc
     */
    synthesis() {
        return __awaiter(this, void 0, void 0, function* () {
            yield EventSubscriberManager.synthesizeEventSubscribers();
        });
    }
}
exports.EventSubscriberManager = EventSubscriberManager;
