"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceContainer = void 0;
// Modules.
const path = require("path");
// Imports.
const Akechi_1 = require("../Confidant/Akechi");
const Igor_1 = require("../Confidant/Igor");
const Morgana_1 = require("../Confidant/Morgana");
const RuntimeProcessId_1 = require("./RuntimeProcessId");
/**
 * Provides a static class with helper functions pertaining to commands.
 */
class ServiceContainer {
    /**
     * Fetch a service from the Core container.
     */
    static get(id) {
        // Find the service.
        const service = ServiceContainer.services.find((prop) => prop.constructor === id || prop.id === id);
        // If the service wasn't found, throw a warning.
        if (!service) {
            // tslint:disable-next-line:comment-format
            // noinspection JSIgnoredPromiseFromCall
            Igor_1.Igor.throw(`Tried to call non-existent service "${id}".`);
            return undefined;
        }
        return service;
    }
    /**
     * Get all services of a specific tag.
     *
     * @param tag
     *   The tag to search for.
     */
    static getServicesWithTag(tag) {
        return ServiceContainer.services.filter((service) => service.tags.includes(tag)) || [];
    }
    /**
     * Check if a service exists in the container.
     *
     * @param id
     *   ID of the service to check.
     *
     * @returns
     *   Returns true if the service is found, false otherwise.
     */
    static serviceExists(id) {
        return ServiceContainer.services.find((prop) => prop.id === id) !== undefined;
    }
    /**
     * Load services defined in a service definition file and append them directly to Lavenza's Core.
     *
     * @param definitionFilePath
     *   Path to the service definition file.
     * @param talent
     *   Talent to link the loaded services to.
     */
    static load(definitionFilePath, talent) {
        return __awaiter(this, void 0, void 0, function* () {
            // First, we'll simply check if the file exists.
            if (!(yield Akechi_1.Akechi.fileExists(definitionFilePath))) {
                yield Morgana_1.Morgana.warn(`No Service Definition File found at "${definitionFilePath}".`);
            }
            // If the file exists, we'll use the yaml loader to get it.
            const definitionFileData = yield Akechi_1.Akechi.readYamlFile(definitionFilePath);
            const serviceDefinitions = definitionFileData.services;
            const runtimeFlow = definitionFileData.runtime;
            // Declare variable to hold services.
            const services = [];
            // For all of the definitions, we want to load services.
            for (const [serviceId, definition] of Object.entries(serviceDefinitions)) {
                // If his service already exists, we should exit with an error.
                if (services.find((srv) => srv.id === serviceId)) {
                    yield Igor_1.Igor.throw(`Duplicate services {${serviceId}}. Please adjust service names accordingly.`);
                }
                // Get the true path to the service, relative to the location of the definition file.
                const serviceFilePath = `${path.dirname(definitionFilePath)}/${definition.class}`;
                // If the service file doesn't exist, we stop here and exit the program completely.
                if (!Akechi_1.Akechi.fileExists(serviceFilePath)) {
                    yield Igor_1.Igor.throw(`The requested service file "${serviceFilePath}" for service {${serviceFilePath}} does not exist.`);
                }
                // We will simply require the file here.
                let service = yield Promise.resolve().then(() => require(serviceFilePath));
                service = new service[path.basename(serviceFilePath, ".js")](serviceId, definition.dependencies, definition.tags, talent);
                // Assign the service to the variable we defined.
                services.push(service);
                // Tasty flavour.
                yield Morgana_1.Morgana.success(`Successfully loaded the {${service.id}} service.`);
            }
            // Set all the services we loaded.
            ServiceContainer.services = [...ServiceContainer.services, ...services];
            // Check Service dependencies and run build functions.
            for (const service of ServiceContainer.services) {
                // Check if all dependencies exist for this service.
                for (const dependency of service.dependencies) {
                    // If the dependency isn't loaded, we exit with an error.
                    if (!ServiceContainer.services.find((srv) => srv.id === dependency)) {
                        yield Igor_1.Igor.throw(`Service {${service.id}} depends on non-existent service {${dependency}}. Please verify that all needed services exist and are instantiated during runtime."`);
                    }
                }
                // Run build tasks.
                yield service.build();
            }
            // Load Runtime Flows.
            for (const [processId, processServices] of Object.entries(runtimeFlow)) {
                // If the process ID is not in the list of those defined in the Enum, we exit with an error.
                if (!(processId in RuntimeProcessId_1.RuntimeProcessId)) {
                    yield Igor_1.Igor.throw(`The defined process "${processId}" is invalid. Please verify your .services.yml files!`);
                }
                // If there are no processServices, we skip to the next item..
                if (!processServices) {
                    continue;
                }
                // Now we'll loop through the defined processes.
                for (const serviceDefinition of processServices) {
                    // If the service doesn't exist, for whatever reason, we should exit.
                    if (!ServiceContainer.serviceExists(serviceDefinition.service)) {
                        yield Igor_1.Igor.throw(`The defined service {${serviceDefinition.service}} does not exist or was not loaded. Please verify your .services.yml files!`);
                    }
                    // Otherwise, we're good.
                    // We add this service to the runtime tasks array.
                    // If the runtime tasks array for this process doesn't exist yet, we initialize it.
                    ServiceContainer.runtimeTasks[processId] = ServiceContainer.runtimeTasks[processId] || [];
                    ServiceContainer.runtimeTasks[processId].push({
                        method: serviceDefinition.method || processId,
                        priority: serviceDefinition.priority || 0,
                        service: ServiceContainer.get(serviceDefinition.service),
                    });
                }
            }
            // Finally, sort all runtime tasks.
            for (const tasks of Object.values(ServiceContainer.runtimeTasks)) {
                tasks.sort(ServiceContainer.prioritySort);
            }
        });
    }
    /**
     * Run tasks for a specified runtime process.
     *
     * @param process
     *   The ID of the runtime process we want to run tasks for.
     */
    static tasks(process) {
        return __awaiter(this, void 0, void 0, function* () {
            // If there are no tasks for this process, we should quit out.
            if (!ServiceContainer.runtimeTasks[process]) {
                yield Morgana_1.Morgana.warn(`There are no runtime tasks specified for the "${process}" process. Proceeding...`);
                return;
            }
            for (const task of ServiceContainer.runtimeTasks[process]) {
                yield Morgana_1.Morgana.status(`TASK ${task.priority}: {${task.service.id}}->${task.method}() ...`);
                yield task.service[task.method]();
                yield Morgana_1.Morgana.success("Done!");
                if (ServiceContainer.runtimeTasks[process].indexOf(task) !== ServiceContainer.runtimeTasks[process].length - 1) {
                    yield Morgana_1.Morgana.status("\r");
                }
            }
        });
    }
    /**
     * Utility function to compare two services using their 'priority' value.
     *
     * @param a
     *   First service.
     * @param b
     *   Second service.
     *
     * @returns
     *   Comparison value.
     *   - If returns less than 0, sort a to an index lower than b (i.e. a comes first).
     *   - If returns 0, leave a and b unchanged with respect to each other,
     *     but sorted with respect to all different elements.
     *   - If returns greater than 0, sort b to an index lower than a (i.e. b comes first).
     */
    static prioritySort(a, b) {
        return b.priority - a.priority;
    }
}
exports.ServiceContainer = ServiceContainer;
/**
 * Store Services.
 */
ServiceContainer.services = [];
/**
 * Store execution flows.
 */
ServiceContainer.runtimeTasks = {};
