"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Talent = void 0;
// Modules.
const path = require("path");
const Akechi_1 = require("../Confidant/Akechi");
const Igor_1 = require("../Confidant/Igor");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const TalentCatalogue_1 = require("./TalentCatalogue");
/**
 * Provides a base class for 'Talents'.
 *
 * 'Talents', in the context of this application, are bundles of functionality that can be granted to any given bot.
 *
 * Think of talents as..."Plugins" from Wordpress, or "Modules" from Drupal, or "Packages" from Laravel.
 *
 * The idea here is that bot features are coded in their own folders and contexts. The power here comes from the
 * flexibility we have since talents can be granted to multiple bots, and talents can be tracked in separate
 * repositories if needed. Also, they can easily be toggled on and off. Decoupling the features from the bots
 * seemed like a good move.
 */
class Talent {
    constructor() {
        /**
         * Object to store relevant paths to the Talent's database entries.
         *
         * Contains paths to Gestalt databases for this Talent.
         *  - databasePaths.global    - Path to global database. i.e. /talents/{TALENT_NAME}
         *  - databasePaths.{BOT_ID}  - Path to bot specific database for this talent. i.e. /bots/{YOUR_BOT}/talents/{TALENT_NAME}
         */
        this.databasePaths = {};
        /**
         * Flag to mark a talent as loaded.
         */
        this.loaded = false;
        /**
         * Priority number of a talent.
         *
         * The Talent's priority rating will affect the order in which certain actions in the application execute.
         */
        this.priority = 0;
    }
    /**
     * Resonate with a message heard in a bot.
     *
     * This is the default implementation of the function. All Talents can choose to resonate and do tasks when messages
     * are heard.
     *
     * @param resonance
     *   The Resonance heard in the bot.
     */
    resonate(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Do nothing by default!
        });
    }
    /**
     * Perform genesis tasks.
     *
     * Each talent will call this function once to set their properties in the genesis phase of the application.
     *
     * @param config
     *   The configuration used to build the Talent. Provided from a 'config.yml' file found in the Talent's folder.
     */
    build(config) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize fields.
            this.machineName = path.basename(config.directory); // Here we get the name of the directory and set it as the ID.
            this.config = config;
            this.priority = config.priority || 0;
            this.directory = config.directory;
        });
    }
    /**
     * Perform any initialization tasks for the Talent, in the context of a given bot.
     *
     * These initialization tasks happen after clients are loaded and authenticated for the bot. This is going to be done
     * in the execution phase.
     *
     * @param bot The bot to perform initializations for.
     */
    initialize(bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Set the path to the talent's bot specific database.
            this.databasePaths[bot.id] = `/bots/${bot.id}/talents/${this.machineName}`;
        });
    }
    /**
     * Load this talent.
     *
     * Loading a talent consists of checking if any dependencies exist and loading any dependent talents as well.
     *
     * If all goes well, services for all talents will be loaded.
     */
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            // If this talent has already been loaded, we can exit.
            if (this.loaded) {
                return;
            }
            // First, we do dependency checks.
            for (const dependency of this.config.dependencies) {
                // Attempt to load the dependency from the catalogue.
                const dependencyTalent = ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalent(dependency);
                // Check if the Talent exists, and exit with an error it if doesn't.
                if (!dependencyTalent) {
                    yield Igor_1.Igor.throw(`The "${this.machineName}" talent has the "${dependencyTalent.machineName}" talent set as a dependency, but this dependency doesn't exist. Please add this talent, or remove it from the talents specified in the bot.`);
                }
                // Otherwise, we load the dependency talent as well.
                yield dependencyTalent.load();
            }
            // Check if a services.yml file exists in this talent.
            if (!(yield Akechi_1.Akechi.fileExists(`${this.directory}/${this.machineName}.services.yml`))) {
                return;
            }
            // Otherwise, use the ServiceContainer to load the specified services.
            yield ServiceContainer_1.ServiceContainer.load(`${this.directory}/${this.machineName}.services.yml`, this.machineName);
            // Mark this talent as loaded.
            this.loaded = true;
        });
    }
}
exports.Talent = Talent;
