"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TalentManager = void 0;
const BotCatalogue_1 = require("../Bot/BotCatalogue");
const Igor_1 = require("../Confidant/Igor");
const Morgana_1 = require("../Confidant/Morgana");
const Sojiro_1 = require("../Confidant/Sojiro");
const Service_1 = require("../Service/Service");
const ServiceContainer_1 = require("../Service/ServiceContainer");
const TalentCatalogue_1 = require("./TalentCatalogue");
/**
 * Provides a Manager Service for Talents.
 *
 * 'Talents', in the context of this application, are bundles of functionality that can be granted to any bot.
 *
 * Think of talents as..."Plugins" from Wordpress, or "Modules" from Drupal, or "Packages" from Laravel.
 *
 * The idea here is that bot features are coded in their own folders. The power here comes from the flexibility we have
 * since talents can be granted to multiple bots, and talents can be tracked in separate repositories if needed. Also,
 * they can easily be toggled on and off.
 *
 * Decoupling the features from the bots seemed like a good move.
 */
class TalentManager extends Service_1.Service {
    /**
     * Validate that the bot has dependencies this talent requires.
     *
     * @param talent
     *   Machine name of the Talent to check dependencies for.
     * @param bot
     *   Bot to validate talent dependencies for.
     */
    static validateTalentDependencies(talent, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check talent's configuration to see if dependencies are loaded into this bot.
            for (const dependency of talent.config.dependencies) {
                // If the dependency isn't found in this bot's config, we shouldn't load this talent.
                if (!bot.config.talents.includes(dependency)) {
                    // Send a warning to the console.
                    yield Morgana_1.Morgana.error("The '{{talent}}' talent requires the '{{parent}}' talent to exist and to be enabled, but this is not the case. It will not be activated for {{bot}}.", {
                        bot: bot.id,
                        parent: dependency,
                        talent: talent.machineName,
                    });
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Runs each Talent's initialize() function to run any preparations for the given bot.
     *
     * @param bot
     *   The Bot that we want to initialize talents for.
     */
    static initializeTalentsForBot(bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Await the processing of all of this bot's talents.
            const talents = yield ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalentsForBot(bot);
            // If the bot has no talents, we can return.
            if (!talents) {
                return;
            }
            // Otherwise, run initialization tasks for all talents in a bot.
            for (const talent of talents) {
                // Run this talent's initialize function for this bot.
                yield talent.initialize(bot);
            }
        });
    }
    /**
     * Grants talents to the Bot.
     *
     * There is a collection of Core talents that all bots will have.
     *
     * Custom Talents are configured in the Bot's configuration file. You must enter the ID (directory name) of
     * the talent in the bot's config so that it can be loaded here.
     *
     * It's important to note that Talent Classes are never stored in the bot. Only the IDs are stored.
     *
     * Talents will always be accessed through the TalentCatalogue itself.
     *
     * @param bot
     *   Bot to grant talents to.
     */
    static grantTalentsToBot(bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check if there are talents set in configuration.
            if (Sojiro_1.Sojiro.isEmpty(bot.config.talents)) {
                yield Morgana_1.Morgana.warn("Talents configuration missing for {{bot}}. The bot will not have any cool features!", { bot: bot.id });
                return;
            }
            // If this is the Master bot, we will grant the Master talent.
            if (bot.isMaster) {
                bot.config.talents.push("master");
            }
            // Alternatively, we'll do a quick check to see if someone is trying to set the master talent in config.
            // This talent should not be set here, and instead is automatically assigned to the master bot.
            if (bot.config.talents.includes("master") && !bot.isMaster) {
                bot.config.talents = Sojiro_1.Sojiro.removeFromArray(bot.config.talents, "master");
            }
            // Await the processing of all talents in the bot's config object.
            for (const talentMachineName of bot.config.talents) {
                // Attempt to load the talent from the catalogue.
                const talent = ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).getTalent(talentMachineName);
                // Check if the Talent exists, and exit it if doesn't.
                if (!talent) {
                    yield Igor_1.Igor.exit(`The "${bot.id}" bot has the "${talentMachineName}" talent enabled for it, but this talent doesn't exist. Please add this talent, or remove it from the talents specified in the bot.`);
                }
                // Await validation of talents configured.
                // This basically checks if the talent entered is valid. Invalid ones are removed from the array.
                if (!(yield TalentManager.validateTalentDependencies(talent, bot))) {
                    bot.config.talents = Sojiro_1.Sojiro.removeFromArray(bot.config.talents, talent.machineName);
                    continue;
                }
                // We will also load this talent.
                // If the talent exists, we want to load it.
                yield talent.load();
                // If all is good, we grant this talent to the bot through assignation in the Catalogue.
                yield ServiceContainer_1.ServiceContainer.get(TalentCatalogue_1.TalentCatalogue).assignTalentToBot(talent, bot);
            }
        });
    }
    /**
     * Genesis handler for the TalentManager Service.
     *
     * We enable talents that need to be enabled. Only talents that are enabled in bots are enabled.
     *
     * @see Core.summon();
     *
     * @inheritDoc
     */
    genesis() {
        return __awaiter(this, void 0, void 0, function* () {
            // From the loaded bots, we want to see which talents we need to enable.
            // This genesis task must run after the BotCatalogue's genesis task.
            for (const bot of ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all()) {
                yield TalentManager.grantTalentsToBot(bot);
            }
        });
    }
    /**
     * Statis handler for the TalentManager.
     *
     * @inheritDoc
     */
    statis() {
        return __awaiter(this, void 0, void 0, function* () {
            // Now we want to loop through all bots in the bot catalogue, and initialize talents set for the bot.
            for (const bot of ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all()) {
                yield TalentManager.initializeTalentsForBot(bot);
            }
        });
    }
}
exports.TalentManager = TalentManager;
