"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientUtilityFactory = void 0;
// Imports.
const ClientType_1 = require("../../../../../lib/Lavenza/Client/ClientType");
const DiscordCommandAuthorizer_1 = require("./Discord/DiscordCommandAuthorizer");
const DiscordCommandHelpHandler_1 = require("./Discord/DiscordCommandHelpHandler");
const DiscordCommandInterpreter_1 = require("./Discord/DiscordCommandInterpreter");
const TwitchCommandAuthorizer_1 = require("./Twitch/TwitchCommandAuthorizer");
const TwitchCommandHelpHandler_1 = require("./Twitch/TwitchCommandHelpHandler");
const TwitchCommandInterpreter_1 = require("./Twitch/TwitchCommandInterpreter");
/**
 * Provides a factory to build all utilities that are client dependent.
 */
class ClientUtilityFactory {
    /**
     * Build the appropriate command interpreter given the provided client type.
     *
     * @param resonance
     *   The resonance we want to build an interpreter for.
     *
     * @returns
     *   The interpreter that was built.
     */
    static buildCommandInterpreter(resonance) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the object.
            let interpreter;
            // Depending on the requested type, we build the appropriate client.
            switch (resonance.client.type) {
                // Create a DiscordClient if the type is Discord.
                case ClientType_1.ClientType.Discord: {
                    interpreter = new DiscordCommandInterpreter_1.DiscordCommandInterpreter();
                    break;
                }
                // Create a TwitchClient if the type is Discord.
                case ClientType_1.ClientType.Twitch: {
                    interpreter = new TwitchCommandInterpreter_1.TwitchCommandInterpreter();
                }
            }
            // Return the interpreter.
            return interpreter;
        });
    }
    /**
     * Build the appropriate command authorizer given the provided client type.
     *
     * @param instruction
     *   The Instruction parsed from a resonance.
     * @returns
     *   Authorizer that was instantiated.
     */
    static buildCommandAuthorizer(instruction) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the object.
            let authorizer;
            // Depending on the requested type, we build the appropriate client.
            switch (instruction.resonance.client.type) {
                // Create a DiscordClient if the type is Discord.
                case ClientType_1.ClientType.Discord: {
                    authorizer = new DiscordCommandAuthorizer_1.DiscordCommandAuthorizer(instruction.command, instruction);
                    break;
                }
                // Create a TwitchClient if the type is Discord.
                case ClientType_1.ClientType.Twitch: {
                    authorizer = new TwitchCommandAuthorizer_1.TwitchCommandAuthorizer(instruction.command, instruction);
                }
            }
            // Run build tasks for authorizer.
            yield authorizer.build();
            // Return the authorizer.
            return authorizer;
        });
    }
    /**
     * Build the appropriate help handler given the provided client type.
     *
     * @param instruction
     *   The instruction we want to build a help handler for.
     *
     * @returns
     *   The interpreter that was built.
     */
    static buildCommandHelpHandler(instruction) {
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the object.
            let helpHandler;
            // Depending on the requested type, we build the appropriate client.
            switch (instruction.resonance.client.type) {
                // Create a DiscordClient if the type is Discord.
                case ClientType_1.ClientType.Discord: {
                    helpHandler = new DiscordCommandHelpHandler_1.DiscordCommandHelpHandler();
                    break;
                }
                // Create a TwitchClient if the type is Discord.
                case ClientType_1.ClientType.Twitch: {
                    helpHandler = new TwitchCommandHelpHandler_1.TwitchCommandHelpHandler();
                }
            }
            // Return the handler.
            return helpHandler;
        });
    }
}
exports.ClientUtilityFactory = ClientUtilityFactory;
