"use strict";
/**
 * Project Lavenza
 * Copyright 2017-2019 Aigachu, All Rights Reserved
 *
 * License: https://github.com/Aigachu/Lavenza-II/blob/master/LICENSE
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandComposer = void 0;
const BotCatalogue_1 = require("../../../../lib/Lavenza/Bot/BotCatalogue");
const Akechi_1 = require("../../../../lib/Lavenza/Confidant/Akechi");
const Sojiro_1 = require("../../../../lib/Lavenza/Confidant/Sojiro");
const Composer_1 = require("../../../../lib/Lavenza/Gestalt/Composer/Composer");
const ServiceContainer_1 = require("../../../../lib/Lavenza/Service/ServiceContainer");
const CommandCatalogue_1 = require("./CommandCatalogue");
/**
 * Provides a main composer for the Gestalt main service.
 */
class CommandComposer extends Composer_1.Composer {
    constructor() {
        super(...arguments);
        /**
         * The priority of the Resonator. This determines the order in which Resonators will resonate.
         */
        this.priority = 4500;
    }
    /**
     * Each composer service must implement this function to determine what to do when the Manager calls them.
     *
     * Composers are made to do database bootstrapping during runtime.
     */
    compose() {
        return __awaiter(this, void 0, void 0, function* () {
            // Run Gestalt handlers for each Bot.
            for (const bot of yield ServiceContainer_1.ServiceContainer.get(BotCatalogue_1.BotCatalogue).all()) {
                // Create a database collection for Commands belonging to a Bot.
                yield this.gestaltService.createCollection(`/bots/${bot.id}/commands`);
                // Await the bootstrapping of Commands data.
                const botCommands = yield ServiceContainer_1.ServiceContainer.get(CommandCatalogue_1.CommandCatalogue).getCommandsForEntity(bot);
                for (const command of botCommands) {
                    // Create a database collection for commands belonging to a Bot.
                    yield this.gestaltService.createCollection(`/bots/${bot.id}/commands/${command.id}`);
                    // Synchronization of data between the Command's default configuration and the database configuration.
                    yield this.gestaltService.sync(command.config, `/bots/${bot.id}/commands/${command.id}/config`);
                }
            }
        });
    }
    /**
     * Get the active configuration from the database for this command, in the context of a Bot.
     *
     * @param command
     *   Command we want to get configurations for.
     * @param bot
     *   The bot context for the configuration we want to fetch. Each bot can have different configuration overrides
     *   for talents.
     *
     * @returns
     *   Returns the configuration fetched from the database.
     */
    getActiveCommandConfigForBot(command, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.gestaltService.get(`/bots/${bot.id}/commands/${command.id}/config`);
        });
    }
    /**
     * Retrieve active client configuration for a specific client in a bot.
     *
     * "Active" configuration refers to the configuration found in the database.
     *
     * @param command
     *   Command we want to get configurations for.
     * @param clientType
     *   The type of client configuration to return for the bot.
     * @param bot
     *   Bot to get this configuration for.
     *
     * @returns
     *   The requested client configuration.
     */
    getActiveCommandClientConfigForBot(command, clientType, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Attempt to get the active configuration from the database.
            const activeConfig = yield this.gestaltService.get(`/bots/${bot.id}/commands/${command.id}/${clientType}`);
            if (!Sojiro_1.Sojiro.isEmpty(activeConfig)) {
                return activeConfig;
            }
            // If we don't find any configurations in the database, we'll fetch it normally and then save it.
            const config = yield this.getCommandClientConfig(command, clientType);
            // Sync it to the database.
            yield this.gestaltService.sync(config, `/bots/${bot.id}/commands/${command.id}/${clientType}`);
            // Return the configuration.
            return config;
        });
    }
    /**
     * Retrieve configuration for a specific client.
     *
     * @param command
     *   Command we want to get configurations for.
     * @param clientType
     *   The type of client configuration to return for the bot.
     *
     * @returns
     *   The requested client configuration.
     */
    getCommandClientConfig(command, clientType) {
        return __awaiter(this, void 0, void 0, function* () {
            // Determine path to client configuration.
            const pathToClientConfig = `${command.directory}/clients/${clientType}.yml`;
            // Attempt to fetch client configuration.
            if (!(yield Akechi_1.Akechi.fileExists(pathToClientConfig))) {
                return undefined;
            }
            // Load configuration since it exists.
            return yield Akechi_1.Akechi.readYamlFile(pathToClientConfig);
        });
    }
    /**
     * Retrieve active parameter configuration for a command in a specific bot.
     *
     * "Active" configuration refers to the configuration found in the database.
     *
     * @param bot
     *   Bot to get this configuration for.
     * @param command
     *   Command we want to get configurations for.
     *
     * @returns
     *   The requested parameter configuration for the given bot obtained frm the database.
     */
    getActiveParameterConfigForBot(command, bot) {
        return __awaiter(this, void 0, void 0, function* () {
            // Attempt to get the active configuration from the database.
            const activeConfig = yield this.gestaltService.get(`/bots/${bot.id}/commands/${command.id}/parameters`);
            if (!Sojiro_1.Sojiro.isEmpty(activeConfig)) {
                return activeConfig;
            }
            // If we don't find any configurations in the database, we'll fetch it normally and then save it.
            const config = yield this.getCommandParameterConfig(command);
            // Sync it to the database.
            yield this.gestaltService.sync(config, `/bots/${bot.id}/commands/${command.id}/parameters`);
            // Return the configuration.
            return config;
        });
    }
    /**
     * Retrieve parameter configuration for a command.
     *
     * @param command
     *   Command we want to get configurations for.
     *
     * @returns
     *   The parameter configuration obtained from the core files.
     */
    getCommandParameterConfig(command) {
        return __awaiter(this, void 0, void 0, function* () {
            // Determine path to client configuration.
            const pathToParameterConfig = `${command.directory}/parameters.yml`;
            // Attempt to fetch client configuration.
            if (!(yield Akechi_1.Akechi.fileExists(pathToParameterConfig))) {
                return {};
            }
            // Load configuration since it exists.
            const config = yield Akechi_1.Akechi.readYamlFile(pathToParameterConfig);
            return Sojiro_1.Sojiro.isEmpty(config) ? {} : config;
        });
    }
}
exports.CommandComposer = CommandComposer;
